@@ -295,36 +295,27 @@ cally_actor_initialize (AtkObject *obj,
   g_object_set_data (G_OBJECT (obj), "atk-component-layer",
                      GINT_TO_POINTER (ATK_LAYER_MDI));
 
-  /* Depends if the object implement ClutterContainer */
-  if (CLUTTER_IS_CONTAINER(actor))
-    {
-      priv->children = clutter_container_get_children (CLUTTER_CONTAINER (actor));
+  priv->children = clutter_actor_get_children (actor);
 
-      /*
-       * We store the handler ids for these signals in case some objects
-       * need to remove these handlers.
-       */
-      handler_id = g_signal_connect (actor,
-                                     "actor-added",
-                                     G_CALLBACK (cally_actor_add_actor),
-                                     obj);
-      g_object_set_data (G_OBJECT (obj), "cally-add-handler-id",
-                         GUINT_TO_POINTER (handler_id));
-      handler_id = g_signal_connect (actor,
-                                     "actor-removed",
-                                     G_CALLBACK (cally_actor_remove_actor),
-                                     obj);
-      g_object_set_data (G_OBJECT (obj), "cally-remove-handler-id",
-                         GUINT_TO_POINTER (handler_id));
+  /*
+   * We store the handler ids for these signals in case some objects
+   * need to remove these handlers.
+   */
+  handler_id = g_signal_connect (actor,
+                                 "actor-added",
+                                 G_CALLBACK (cally_actor_add_actor),
+                                 obj);
+  g_object_set_data (G_OBJECT (obj), "cally-add-handler-id",
+                     GUINT_TO_POINTER (handler_id));
+  handler_id = g_signal_connect (actor,
+                                 "actor-removed",
+                                 G_CALLBACK (cally_actor_remove_actor),
+                                 obj);
+  g_object_set_data (G_OBJECT (obj), "cally-remove-handler-id",
+                     GUINT_TO_POINTER (handler_id));
 
-      obj->role = ATK_ROLE_PANEL; /* typically objects implementing ClutterContainer
-                                     interface would be a panel */
-    }
-  else
-    {
-      priv->children = NULL;
-      obj->role = ATK_ROLE_UNKNOWN;
-    }
+  obj->role = ATK_ROLE_PANEL; /* typically objects implementing ClutterContainer
+                                 interface would be a panel */
 }
 
 static void
@@ -462,11 +453,11 @@ cally_actor_get_parent (AtkObject *obj)
 static gint
 cally_actor_get_index_in_parent (AtkObject *obj)
 {
-  CallyActor    *cally_actor   = NULL;
-  ClutterActor *actor        = NULL;
+  CallyActor *cally_actor = NULL;
+  ClutterActor *actor = NULL;
   ClutterActor *parent_actor = NULL;
-  GList        *children     = NULL;
-  gint          index        = -1;
+  ClutterActor *iter;
+  gint index = -1;
 
   g_return_val_if_fail (CALLY_IS_ACTOR (obj), -1);
 
@@ -496,14 +487,17 @@ cally_actor_get_index_in_parent (AtkObject *obj)
   if (actor == NULL) /* Object is defunct */
     return -1;
 
-  parent_actor = clutter_actor_get_parent(actor);
-  if ((parent_actor == NULL)||(!CLUTTER_IS_CONTAINER(parent_actor)))
+  index = 0;
+  parent_actor = clutter_actor_get_parent (actor);
+  if (parent_actor == NULL)
     return -1;
 
-  children = clutter_container_get_children(CLUTTER_CONTAINER(parent_actor));
-
-  index = g_list_index (children, actor);
-  g_list_free (children);
+  for (iter = clutter_actor_get_first_child (parent_actor);
+       iter != NULL && iter != actor;
+       iter = clutter_actor_get_next_sibling (iter))
+    {
+      index += 1;
+    }
 
   return index;
 }
@@ -565,9 +559,7 @@ cally_actor_ref_state_set (AtkObject *obj)
 static gint
 cally_actor_get_n_children (AtkObject *obj)
 {
-  ClutterActor     *actor    = NULL;
-  GList            *children = NULL;
-  gint              num      = 0;
+  ClutterActor *actor = NULL;
 
   g_return_val_if_fail (CALLY_IS_ACTOR (obj), 0);
 
@@ -578,57 +570,32 @@ cally_actor_get_n_children (AtkObject *obj)
 
   g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), 0);
 
-  if (CLUTTER_IS_CONTAINER (actor))
-    {
-      children = clutter_container_get_children (CLUTTER_CONTAINER (actor));
-      num = g_list_length (children);
-
-      g_list_free (children);
-    }
-  else
-    {
-      num = 0;
-    }
-
-  return num;
+  return clutter_actor_get_n_children (actor);
 }
 
 static AtkObject*
 cally_actor_ref_child (AtkObject *obj,
-                      gint       i)
+                       gint       i)
 {
-  ClutterActor     *actor    = NULL;
-  ClutterActor     *child    = NULL;
-  GList            *children = NULL;
-  AtkObject        *result   = NULL;
+  ClutterActor *actor = NULL;
+  ClutterActor *child = NULL;
 
   g_return_val_if_fail (CALLY_IS_ACTOR (obj), NULL);
 
   actor = CALLY_GET_CLUTTER_ACTOR (obj);
-
   if (actor == NULL) /* State is defunct */
-    {
-      return NULL;
-    }
+    return NULL;
 
   g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), NULL);
 
-  if (CLUTTER_IS_CONTAINER (actor))
-    {
-      children = clutter_container_get_children (CLUTTER_CONTAINER (actor));
-      child = g_list_nth_data (children, i);
+  if (i >= clutter_actor_get_n_children (actor))
+    return NULL;
 
-      result = clutter_actor_get_accessible (child);
+  child = clutter_actor_get_child_at_index (actor, i);
+  if (child == NULL)
+    return NULL;
 
-      g_object_ref (result);
-      g_list_free (children);
-    }
-  else
-    {
-      result = NULL;
-    }
-
-  return result;
+  return g_object_ref (clutter_actor_get_accessible (child));
 }
 
 static AtkAttributeSet *
@@ -682,8 +649,8 @@ cally_actor_remove_actor (ClutterActor *container,
 
 static gint
 cally_actor_real_add_actor (ClutterActor *container,
-                           ClutterActor *actor,
-                           gpointer      data)
+                            ClutterActor *actor,
+                            gpointer      data)
 {
   AtkObject        *atk_parent = ATK_OBJECT (data);
   AtkObject        *atk_child  = clutter_actor_get_accessible (actor);
@@ -698,8 +665,7 @@ cally_actor_real_add_actor (ClutterActor *container,
 
   g_list_free (priv->children);
 
-  priv->children =
-    clutter_container_get_children (CLUTTER_CONTAINER(container));
+  priv->children = clutter_actor_get_children (CLUTTER_ACTOR (container));
 
   index = g_list_index (priv->children, actor);
   g_signal_emit_by_name (atk_parent, "children_changed::add",
@@ -710,8 +676,8 @@ cally_actor_real_add_actor (ClutterActor *container,
 
 static gint
 cally_actor_real_remove_actor (ClutterActor *container,
-                              ClutterActor *actor,
-                              gpointer      data)
+                               ClutterActor *actor,
+                               gpointer      data)
 {
   AtkPropertyValues  values      = { NULL };
   AtkObject*         atk_parent  = NULL;
@@ -741,7 +707,8 @@ cally_actor_real_remove_actor (ClutterActor *container,
   priv = CALLY_ACTOR (atk_parent)->priv;
   index = g_list_index (priv->children, actor);
   g_list_free (priv->children);
-  priv->children = clutter_container_get_children (CLUTTER_CONTAINER(container));
+
+  priv->children = clutter_actor_get_children (CLUTTER_ACTOR (container));
 
   if (index >= 0 && index <= g_list_length (priv->children))
     g_signal_emit_by_name (atk_parent, "children_changed::remove",
