@@ -119,13 +119,15 @@ struct _ClutterActor
  * @show_all: virtual function for containers and composite actors, to
  *   determine which children should be shown when calling
  *   clutter_actor_show_all() on the actor. Defaults to calling
- *   clutter_actor_show().
+ *   clutter_actor_show(). This virtual function is deprecated and it
+ *   should not be overridden.
  * @hide: signal class handler for #ClutterActor::hide; it must chain
  *   up to the parent's implementation
  * @hide_all: virtual function for containers and composite actors, to
  *   determine which children should be shown when calling
  *   clutter_actor_hide_all() on the actor. Defaults to calling
- *   clutter_actor_hide().
+ *   clutter_actor_hide(). This virtual function is deprecated and it
+ *   should not be overridden.
  * @realize: virtual function, used to allocate resources for the actor;
  *   it should chain up to the parent's implementation
  * @unrealize: virtual function, used to deallocate resources allocated
@@ -272,9 +274,7 @@ void                  clutter_actor_unset_flags               (ClutterActor
 ClutterActorFlags     clutter_actor_get_flags                 (ClutterActor          *self);
 
 void                  clutter_actor_show                      (ClutterActor          *self);
-void                  clutter_actor_show_all                  (ClutterActor          *self);
 void                  clutter_actor_hide                      (ClutterActor          *self);
-void                  clutter_actor_hide_all                  (ClutterActor          *self);
 void                  clutter_actor_realize                   (ClutterActor          *self);
 void                  clutter_actor_unrealize                 (ClutterActor          *self);
 void                  clutter_actor_map                       (ClutterActor          *self);
@@ -323,6 +323,9 @@ void                  clutter_actor_allocate_align_fill       (ClutterActor
                                                                gboolean               x_fill,
                                                                gboolean               y_fill,
                                                                ClutterAllocationFlags flags);
+void                  clutter_actor_set_allocation            (ClutterActor          *self,
+                                                               const ClutterActorBox *box,
+                                                               ClutterAllocationFlags flags);
 void                  clutter_actor_get_allocation_box        (ClutterActor          *self,
                                                                ClutterActorBox       *box);
 void                  clutter_actor_get_allocation_geometry   (ClutterActor          *self,
