@@ -4,8 +4,6 @@
 #define __CLUTTER_DEPRECATED_H_INSIDE__
 
 #include "deprecated/clutter-actor.h"
-#include "deprecated/clutter-animatable.h"
-#include "deprecated/clutter-animation.h"
 #include "deprecated/clutter-backend.h"
 #include "deprecated/clutter-behaviour.h"
 #include "deprecated/clutter-behaviour-depth.h"
@@ -15,7 +13,6 @@
 #include "deprecated/clutter-behaviour-rotate.h"
 #include "deprecated/clutter-behaviour-scale.h"
 #include "deprecated/clutter-box.h"
-#include "deprecated/clutter-cairo-texture.h"
 #include "deprecated/clutter-container.h"
 #include "deprecated/clutter-fixed.h"
 #include "deprecated/clutter-frame-source.h"
@@ -25,12 +22,8 @@
 #include "deprecated/clutter-rectangle.h"
 #include "deprecated/clutter-score.h"
 #include "deprecated/clutter-shader.h"
-#include "deprecated/clutter-stage-manager.h"
 #include "deprecated/clutter-stage.h"
-#include "deprecated/clutter-texture.h"
-#include "deprecated/clutter-timeline.h"
 #include "deprecated/clutter-timeout-pool.h"
-#include "deprecated/clutter-util.h"
 
 #undef __CLUTTER_DEPRECATED_H_INSIDE__
 
