@@ -73,6 +73,9 @@
 
 #include <math.h>
 
+#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
+#include "deprecated/clutter-container.h"
+
 #include "clutter-actor.h"
 #include "clutter-animatable.h"
 #include "clutter-child-meta.h"
@@ -202,11 +205,11 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
                                          gfloat               *nat_width_p)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
-  GList *l, *children = clutter_container_get_children (container);
   gint n_rows, line_item_count, line_count;
   gfloat total_min_width, total_natural_width;
   gfloat line_min_width, line_natural_width;
   gfloat max_min_width, max_natural_width;
+  ClutterActor *actor, *child;
   gfloat item_y;
 
   n_rows = get_rows (CLUTTER_FLOW_LAYOUT (manager), for_height);
@@ -222,6 +225,8 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
 
   item_y = 0;
 
+  actor = CLUTTER_ACTOR (container);
+
   /* clear the line width arrays */
   if (priv->line_min != NULL)
     g_array_free (priv->line_min, TRUE);
@@ -236,14 +241,15 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
                                           sizeof (gfloat),
                                           16);
 
-  if (children)
+  if (clutter_actor_get_n_children (actor) != 0)
     line_count = 1;
 
   max_min_width = max_natural_width = 0;
 
-  for (l = children; l != NULL; l = l->next)
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       gfloat child_min, child_natural;
       gfloat new_y, item_height;
 
@@ -301,8 +307,6 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
         }
     }
 
-  g_list_free (children);
-
   priv->col_width = max_natural_width;
 
   if (priv->max_col_width > 0 && priv->col_width > priv->max_col_width)
@@ -365,7 +369,7 @@ clutter_flow_layout_get_preferred_width (ClutterLayoutManager *manager,
   priv->req_height = for_height;
 
   if (min_width_p)
-    *min_width_p = total_min_width;
+    *min_width_p = max_min_width;
 
   if (nat_width_p)
     *nat_width_p = total_natural_width;
@@ -379,11 +383,11 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
                                           gfloat               *nat_height_p)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
-  GList *l, *children = clutter_container_get_children (container);
   gint n_columns, line_item_count, line_count;
   gfloat total_min_height, total_natural_height;
   gfloat line_min_height, line_natural_height;
   gfloat max_min_height, max_natural_height;
+  ClutterActor *actor, *child;
   gfloat item_x;
 
   n_columns = get_columns (CLUTTER_FLOW_LAYOUT (manager), for_width);
@@ -399,6 +403,8 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
 
   item_x = 0;
 
+  actor = CLUTTER_ACTOR (container);
+
   /* clear the line height arrays */
   if (priv->line_min != NULL)
     g_array_free (priv->line_min, TRUE);
@@ -413,14 +419,15 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
                                           sizeof (gfloat),
                                           16);
 
-  if (children)
+  if (clutter_actor_get_n_children (actor) != 0)
     line_count = 1;
 
   max_min_height = max_natural_height = 0;
 
-  for (l = children; l != NULL; l = l->next)
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       gfloat child_min, child_natural;
       gfloat new_x, item_width;
 
@@ -479,8 +486,6 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
         }
     }
 
-  g_list_free (children);
-
   priv->row_height = max_natural_height;
 
   if (priv->max_row_height > 0 && priv->row_height > priv->max_row_height)
@@ -542,7 +547,7 @@ clutter_flow_layout_get_preferred_height (ClutterLayoutManager *manager,
   priv->req_width = for_width;
 
   if (min_height_p)
-    *min_height_p = total_min_height;
+    *min_height_p = max_min_height;
 
   if (nat_height_p)
     *nat_height_p = total_natural_height;
@@ -555,16 +560,19 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
                               ClutterAllocationFlags  flags)
 {
   ClutterFlowLayoutPrivate *priv = CLUTTER_FLOW_LAYOUT (manager)->priv;
-  GList *l, *children = clutter_container_get_children (container);
+  ClutterActor *actor, *child;
+  gfloat x_off, y_off;
   gfloat avail_width, avail_height;
   gfloat item_x, item_y;
   gint line_item_count;
   gint items_per_line;
   gint line_index;
 
-  if (children == NULL)
+  actor = CLUTTER_ACTOR (container);
+  if (clutter_actor_get_n_children (actor) == 0)
     return;
 
+  clutter_actor_box_get_origin (allocation, &x_off, &y_off);
   clutter_actor_box_get_size (allocation, &avail_width, &avail_height);
 
   /* blow the cached preferred size and re-compute with the given
@@ -585,14 +593,16 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
   items_per_line = compute_lines (CLUTTER_FLOW_LAYOUT (manager),
                                   avail_width, avail_height);
 
-  item_x = item_y = 0;
+  item_x = x_off;
+  item_y = y_off;
 
   line_item_count = 0;
   line_index = 0;
 
-  for (l = children; l != NULL; l = l->next)
+  for (child = clutter_actor_get_first_child (actor);
+       child != NULL;
+       child = clutter_actor_get_next_sibling (child))
     {
-      ClutterActor *child = l->data;
       ClutterActorBox child_alloc;
       gfloat item_width, item_height;
       gfloat new_x, new_y;
@@ -616,10 +626,10 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_x = 0;
+              item_x = x_off;
             }
 
-          new_x = ((line_item_count + 1) * (avail_width + priv->col_spacing))
+          new_x = x_off + ((line_item_count + 1) * (avail_width + priv->col_spacing))
                 / items_per_line;
           item_width = new_x - item_x - priv->col_spacing;
           item_height = g_array_index (priv->line_natural,
@@ -655,10 +665,10 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
               line_item_count = 0;
               line_index += 1;
 
-              item_y = 0;
+              item_y = y_off;
             }
 
-          new_y = ((line_item_count + 1) * (avail_height + priv->row_spacing))
+          new_y = y_off + ((line_item_count + 1) * (avail_height + priv->row_spacing))
                 / items_per_line;
           item_height = new_y - item_y - priv->row_spacing;
           item_width = g_array_index (priv->line_natural,
@@ -700,8 +710,6 @@ clutter_flow_layout_allocate (ClutterLayoutManager   *manager,
 
       line_item_count += 1;
     }
-
-  g_list_free (children);
 }
 
 static void
