@@ -28,6 +28,8 @@
 #ifndef __CLUTTER_TYPES_H__
 #define __CLUTTER_TYPES_H__
 
+#include <cairo.h>
+
 #include <clutter/clutter-enums.h>
 
 G_BEGIN_DECLS
@@ -47,9 +49,12 @@ G_BEGIN_DECLS
 #endif
 
 #define CLUTTER_TYPE_ACTOR_BOX          (clutter_actor_box_get_type ())
+#define CLUTTER_TYPE_FOG                (clutter_fog_get_type ())
 #define CLUTTER_TYPE_GEOMETRY           (clutter_geometry_get_type ())
 #define CLUTTER_TYPE_KNOT               (clutter_knot_get_type ())
+#define CLUTTER_TYPE_MARGIN             (clutter_margin_get_type ())
 #define CLUTTER_TYPE_PAINT_VOLUME       (clutter_paint_volume_get_type ())
+#define CLUTTER_TYPE_PERSPECTIVE        (clutter_perspective_get_type ())
 #define CLUTTER_TYPE_VERTEX             (clutter_vertex_get_type ())
 
 typedef struct _ClutterActor            ClutterActor;
@@ -59,6 +64,7 @@ typedef struct _ClutterContainer        ClutterContainer; /* dummy */
 typedef struct _ClutterChildMeta        ClutterChildMeta;
 typedef struct _ClutterLayoutMeta       ClutterLayoutMeta;
 typedef struct _ClutterActorMeta        ClutterActorMeta;
+typedef struct _ClutterLayoutManager    ClutterLayoutManager;
 
 typedef struct _ClutterAlpha            ClutterAlpha;
 typedef struct _ClutterAnimatable       ClutterAnimatable; /* dummy */
@@ -74,8 +80,11 @@ typedef struct _ClutterPath             ClutterPath;
 
 typedef struct _ClutterActorBox         ClutterActorBox;
 typedef struct _ClutterColor            ClutterColor;
+typedef struct _ClutterFog              ClutterFog;
 typedef struct _ClutterGeometry         ClutterGeometry;
 typedef struct _ClutterKnot             ClutterKnot;
+typedef struct _ClutterMargin           ClutterMargin;
+typedef struct _ClutterPerspective      ClutterPerspective;
 typedef struct _ClutterVertex           ClutterVertex;
 
 typedef struct _ClutterBehaviour        ClutterBehaviour;
@@ -258,6 +267,31 @@ void                clutter_paint_volume_union               (ClutterPaintVolume
 gboolean            clutter_paint_volume_set_from_allocation (ClutterPaintVolume       *pv,
                                                               ClutterActor             *actor);
 
+/**
+ * ClutterMargin:
+ * @left: the margin from the left
+ * @right: the margin from the right
+ * @top: the margin from the top
+ * @bottom: the margin from the bottom
+ *
+ * A representation of the components of a margin.
+ *
+ * Since: 1.10
+ */
+struct _ClutterMargin
+{
+  float left;
+  float right;
+  float top;
+  float bottom;
+};
+
+GType clutter_margin_get_type (void) G_GNUC_CONST;
+
+ClutterMargin * clutter_margin_new      (void) G_GNUC_MALLOC;
+ClutterMargin * clutter_margin_copy     (const ClutterMargin *margin_);
+void            clutter_margin_free     (ClutterMargin       *margin_);
+
 G_END_DECLS
 
 #endif /* __CLUTTER_TYPES_H__ */
