@@ -34,7 +34,6 @@
 #include "clutter-actor-private.h"
 #include "clutter-backend-private.h"
 #include "clutter-debug.h"
-#include "clutter-device-manager-private.h"
 #include "clutter-event-private.h"
 #include "clutter-main.h"
 #include "clutter-paint-volume-private.h"
@@ -79,12 +78,9 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
   if (gdk_event->any.window == NULL)
     return GDK_FILTER_CONTINUE;
 
-  device_manager = clutter_device_manager_get_default ();
-  if (G_UNLIKELY (device_manager == NULL))
-    return GDK_FILTER_CONTINUE;
-
   backend_gdk = CLUTTER_BACKEND_GDK (backend);
   stage = clutter_gdk_get_stage_from_window (gdk_event->any.window);
+  device_manager = clutter_device_manager_get_default ();
 
   gdk_device = gdk_event_get_device (gdk_event);
   if (gdk_device != NULL)
@@ -103,7 +99,7 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
   if (stage == NULL)
     return GDK_FILTER_CONTINUE;
 
-  _clutter_threads_acquire_lock ();
+  clutter_threads_enter ();
 
   switch (gdk_event->type)
     {
@@ -184,33 +180,6 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
                     event->button.y);
       break;
 
-    case GDK_TOUCH_BEGIN:
-    case GDK_TOUCH_END:
-    case GDK_TOUCH_CANCEL:
-    case GDK_TOUCH_UPDATE:
-      event = clutter_event_new (gdk_event->type == GDK_TOUCH_BEGIN ?
-                                 CLUTTER_TOUCH_BEGIN :
-                                 ((gdk_event->type == GDK_TOUCH_END) ?
-                                  CLUTTER_TOUCH_END :
-                                  (gdk_event->type == GDK_TOUCH_UPDATE ?
-                                   CLUTTER_TOUCH_UPDATE :
-                                   CLUTTER_TOUCH_CANCEL)));
-      event->touch.time = gdk_event->touch.time;
-      event->touch.x = gdk_event->touch.x;
-      event->touch.y = gdk_event->touch.y;
-      event->touch.sequence = (ClutterEventSequence *) gdk_event->touch.sequence;
-      event->touch.modifier_state = gdk_event->touch.state;
-      clutter_event_set_device (event, device);
-      clutter_event_set_source_device (event, source_device);
-      CLUTTER_NOTE (EVENT, "Touch %p %s [%",
-                    event->touch.sequence,
-                    event->type == CLUTTER_TOUCH_BEGIN ? "begin" :
-                    (event->type == CLUTTER_TOUCH_END ? "end" :
-                     (event->type == CLUTTER_TOUCH_UPDATE ? "update"
-                      : "cancel")),
-                    event->touch.x, event->touch.y);
-      break;
-
     case GDK_2BUTTON_PRESS:
     case GDK_3BUTTON_PRESS:
       /* these are handled by clutter-main.c updating click_count */
@@ -247,12 +216,9 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
       clutter_event_set_device (event, device);
       clutter_event_set_source_device (event, source_device);
       if (gdk_event->type == GDK_ENTER_NOTIFY)
-        _clutter_input_device_set_stage (clutter_event_get_device (event), stage);
-      else if (gdk_event->type == GDK_LEAVE_NOTIFY &&
-               gdk_event->crossing.mode != GDK_CROSSING_TOUCH_BEGIN &&
-               gdk_event->crossing.mode != GDK_CROSSING_TOUCH_END &&
-               gdk_event->crossing.mode != GDK_CROSSING_DEVICE_SWITCH)
-        _clutter_input_device_set_stage (clutter_event_get_device (event), NULL);
+        _clutter_stage_add_device (stage, clutter_event_get_device (event));
+      else
+        _clutter_stage_remove_device (stage, clutter_event_get_device (event));
       CLUTTER_NOTE (EVENT, "Crossing %s [%.2f, %.2f]",
                     event->type == CLUTTER_ENTER ? "enter" : "leave",
                     event->crossing.x,
@@ -260,10 +226,12 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
       break;
 
     case GDK_FOCUS_CHANGE:
-      if (gdk_event->focus_change.in)
-        _clutter_stage_update_state (stage, 0, CLUTTER_STAGE_STATE_ACTIVATED);
-      else
-        _clutter_stage_update_state (stage, CLUTTER_STAGE_STATE_ACTIVATED, 0);
+      event = clutter_event_new (CLUTTER_STAGE_STATE);
+      event->stage_state.time = 0; /* XXX: there is no timestamp in this GdkEvent */
+      event->stage_state.changed_mask = CLUTTER_STAGE_STATE_ACTIVATED;
+      event->stage_state.new_state = gdk_event->focus_change.in
+                                   ? CLUTTER_STAGE_STATE_ACTIVATED
+                                   : 0;
       break;
 
     case GDK_CONFIGURE:
@@ -272,8 +240,8 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
 
         clutter_actor_get_size (CLUTTER_ACTOR (stage), &w, &h);
 
-        if ((int) w != gdk_event->configure.width ||
-            (int) h != gdk_event->configure.height)
+        if (w != gdk_event->configure.width ||
+            h != gdk_event->configure.height)
           {
             clutter_actor_set_size (CLUTTER_ACTOR (stage),
                                     gdk_event->configure.width,
@@ -289,25 +257,29 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
       event->scroll.y = gdk_event->scroll.y;
       event->scroll.modifier_state = gdk_event->scroll.state;
       event->scroll.axes = NULL;
-      /* XXX: must keep ClutterScrollDirection compatible with GdkScrollDirection */
-      event->scroll.direction = (ClutterScrollDirection) gdk_event->scroll.direction;
+      event->scroll.direction = gdk_event->scroll.direction;
       clutter_event_set_device (event, device);
       clutter_event_set_source_device (event, source_device);
-      clutter_event_set_scroll_delta (event,
-                                      gdk_event->scroll.delta_x,
-                                      gdk_event->scroll.delta_y);
       break;
 
     case GDK_WINDOW_STATE:
+      event = clutter_event_new (CLUTTER_STAGE_STATE);
+      event->stage_state.changed_mask = 0;
+      event->stage_state.new_state = 0;
+      if (gdk_event->window_state.changed_mask & GDK_WINDOW_STATE_WITHDRAWN)
+        {
+          event->stage_state.changed_mask |= CLUTTER_STAGE_STATE_OFFSCREEN;
+          event->stage_state.new_state |= (gdk_event->window_state.new_window_state & GDK_WINDOW_STATE_WITHDRAWN)
+                                        ? CLUTTER_STAGE_STATE_OFFSCREEN
+                                        : 0;
+        }
+
       if (gdk_event->window_state.changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
         {
-          gboolean is_fullscreen;
-
-          is_fullscreen = (gdk_event->window_state.new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0;
-          if (is_fullscreen)
-            _clutter_stage_update_state (stage, 0, CLUTTER_STAGE_STATE_FULLSCREEN);
-          else
-            _clutter_stage_update_state (stage, CLUTTER_STAGE_STATE_FULLSCREEN, 0);
+          event->stage_state.changed_mask |= CLUTTER_STAGE_STATE_FULLSCREEN;
+          event->stage_state.new_state |= (gdk_event->window_state.new_window_state & GDK_WINDOW_STATE_FULLSCREEN)
+                                        ? CLUTTER_STAGE_STATE_FULLSCREEN
+                                        : 0;
         }
       break;
 
@@ -339,14 +311,15 @@ clutter_gdk_handle_event (GdkEvent *gdk_event)
       while (spin > 0 && (event = clutter_event_get ()))
 	{
 	  /* forward the event into clutter for emission etc. */
-          _clutter_stage_queue_event (event->any.stage, event, FALSE);
+	  clutter_do_event (event);
+	  clutter_event_free (event);
 	  --spin;
 	}
 
       result = GDK_FILTER_REMOVE;
     }
 
-  _clutter_threads_release_lock ();
+  clutter_threads_leave ();
 
   return result;
 }
