@@ -42,26 +42,21 @@
 #include "clutter-stage-private.h"
 
 #include "wayland/clutter-backend-wayland.h"
-#include "wayland/clutter-backend-wayland-priv.h"
 #include "wayland/clutter-device-manager-wayland.h"
 #include "wayland/clutter-event-wayland.h"
 #include "wayland/clutter-stage-wayland.h"
-#include "wayland/clutter-wayland.h"
 #include "cogl/clutter-stage-cogl.h"
 
 #include <wayland-client.h>
-#include <wayland-cursor.h>
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <cogl/cogl.h>
-#include <cogl/cogl-wayland-client.h>
 
 #define clutter_backend_wayland_get_type     _clutter_backend_wayland_get_type
 
 G_DEFINE_TYPE (ClutterBackendWayland, clutter_backend_wayland, CLUTTER_TYPE_BACKEND);
 
-static struct wl_display *_foreign_display = NULL;
-static gboolean _no_event_dispatch = FALSE;
+static void clutter_backend_wayland_load_cursor (ClutterBackendWayland *backend_wayland);
 
 static void
 clutter_backend_wayland_dispose (GObject *gobject)
@@ -80,67 +75,24 @@ clutter_backend_wayland_dispose (GObject *gobject)
       backend_wayland->cursor_buffer = NULL;
     }
 
-  if (backend_wayland->cursor_theme)
-    {
-      wl_cursor_theme_destroy (backend_wayland->cursor_theme);
-      backend_wayland->cursor_theme = NULL;
-    }
-
   G_OBJECT_CLASS (clutter_backend_wayland_parent_class)->dispose (gobject);
 }
 
 
-static void
-output_handle_mode (void             *data,
-                    struct wl_output *wl_output,
-                    uint32_t          flags,
-                    int               width,
-                    int               height,
-                    int               refresh)
-{
-  ClutterBackendWayland *backend_wayland = data;
-
-  if (flags & WL_OUTPUT_MODE_CURRENT)
-    {
-      backend_wayland->output_width = width;
-      backend_wayland->output_height = height;
-    }
-}
 
 static void
-output_handle_geometry (void             *data,
-                        struct wl_output *wl_output,
-                        int               x,
-                        int               y,
-                        int               physical_width,
-                        int               physical_height,
-                        int               subpixel,
-                        const char       *make,
-                        const char       *model,
-                        int32_t transform)
-{
-}
-
-
-static const struct wl_output_listener wayland_output_listener = {
-  output_handle_geometry,
-  output_handle_mode,
-};
-
-
-static void
-registry_handle_global (void *data,
-                        struct wl_registry *registry,
-                        uint32_t id,
-                        const char *interface,
-                        uint32_t version)
+display_handle_global (struct wl_display *display,
+                       uint32_t id,
+                       const char *interface,
+                       uint32_t version,
+                       void *data)
 {
   ClutterBackendWayland *backend_wayland = data;
 
   if (strcmp (interface, "wl_compositor") == 0)
     backend_wayland->wayland_compositor =
-      wl_registry_bind (registry, id, &wl_compositor_interface, 1);
-  else if (strcmp (interface, "wl_seat") == 0)
+      wl_display_bind (display, id, &wl_compositor_interface);
+  else if (strcmp (interface, "wl_input_device") == 0)
     {
       ClutterDeviceManager *device_manager = backend_wayland->device_manager;
       _clutter_device_manager_wayland_add_input_group (device_manager, id);
@@ -148,28 +100,14 @@ registry_handle_global (void *data,
   else if (strcmp (interface, "wl_shell") == 0)
     {
       backend_wayland->wayland_shell =
-        wl_registry_bind (registry, id, &wl_shell_interface, 1);
+        wl_display_bind (display, id, &wl_shell_interface);
+
     }
   else if (strcmp (interface, "wl_shm") == 0)
-    {
-      backend_wayland->wayland_shm =
-        wl_registry_bind (registry, id, &wl_shm_interface, 1);
-    }
-  else if (strcmp (interface, "wl_output") == 0)
-    {
-      /* FIXME: Support multiple outputs */
-      backend_wayland->wayland_output =
-        wl_registry_bind (registry, id, &wl_output_interface, 1);
-      wl_output_add_listener (backend_wayland->wayland_output,
-                              &wayland_output_listener,
-                              backend_wayland);
-    }
+    backend_wayland->wayland_shm =
+      wl_display_bind (display, id, &wl_shm_interface);
 }
 
-static const struct wl_registry_listener wayland_registry_listener = {
-  registry_handle_global,
-};
-
 static gboolean
 clutter_backend_wayland_post_parse (ClutterBackend  *backend,
                                     GError         **error)
@@ -177,10 +115,7 @@ clutter_backend_wayland_post_parse (ClutterBackend  *backend,
   ClutterBackendWayland *backend_wayland = CLUTTER_BACKEND_WAYLAND (backend);
 
   /* TODO: expose environment variable/commandline option for this... */
-  backend_wayland->wayland_display = _foreign_display;
-  if (backend_wayland->wayland_display == NULL)
-    backend_wayland->wayland_display = wl_display_connect (NULL);
-
+  backend_wayland->wayland_display = wl_display_connect (NULL);
   if (!backend_wayland->wayland_display)
     {
       g_set_error (error, CLUTTER_INIT_ERROR,
@@ -189,15 +124,10 @@ clutter_backend_wayland_post_parse (ClutterBackend  *backend,
       return FALSE;
     }
 
-  backend_wayland->wayland_registry =
-    wl_display_get_registry (backend_wayland->wayland_display);
-
   backend_wayland->wayland_source =
     _clutter_event_source_wayland_new (backend_wayland->wayland_display);
   g_source_attach (backend_wayland->wayland_source, NULL);
 
-  g_object_set (clutter_settings_get_default (), "font-dpi", 96 * 1024, NULL);
-
   /* XXX: We require the device manager to exist as soon as we connect to the
    * compositor and setup an event handler because we will immediately be
    * notified of the available input devices which need to be associated with
@@ -213,15 +143,18 @@ clutter_backend_wayland_post_parse (ClutterBackend  *backend,
     _clutter_device_manager_wayland_new (backend);
 
   /* Set up listener so we'll catch all events. */
-  wl_registry_add_listener (backend_wayland->wayland_registry,
-                            &wayland_registry_listener,
-                            backend_wayland);
+  wl_display_add_global_listener (backend_wayland->wayland_display,
+                                  display_handle_global,
+                                  backend_wayland);
 
   /* Wait until we have been notified about the compositor and shell objects */
   while (!(backend_wayland->wayland_compositor &&
            backend_wayland->wayland_shell))
     wl_display_roundtrip (backend_wayland->wayland_display);
 
+  /* We need the shm object before we can create the cursor */
+  clutter_backend_wayland_load_cursor (backend_wayland);
+
   return TRUE;
 }
 
@@ -236,11 +169,12 @@ clutter_backend_wayland_get_renderer (ClutterBackend  *backend,
 
   renderer = cogl_renderer_new ();
 
-  cogl_wayland_renderer_set_event_dispatch_enabled (renderer, !_no_event_dispatch);
-  cogl_renderer_set_winsys_id (renderer, COGL_WINSYS_ID_EGL_WAYLAND);
-
   cogl_wayland_renderer_set_foreign_display (renderer,
                                              backend_wayland->wayland_display);
+  cogl_wayland_renderer_set_foreign_compositor (renderer,
+                                                backend_wayland->wayland_compositor);
+  cogl_wayland_renderer_set_foreign_shell (renderer,
+                                           backend_wayland->wayland_shell);
 
   return renderer;
 }
@@ -291,89 +225,173 @@ clutter_backend_wayland_class_init (ClutterBackendWaylandClass *klass)
   backend_class->get_display = clutter_backend_wayland_get_display;
 }
 
-void
-_clutter_backend_wayland_ensure_cursor (ClutterBackendWayland *backend_wayland)
+/*
+ * clutter_backend_wayland_load_cursor and the two functions below were copied
+ * from GTK+ and adapted for clutter
+ */
+static void
+set_pixbuf (GdkPixbuf     *pixbuf,
+            unsigned char *map,
+            int            width,
+            int            height)
 {
-  struct wl_cursor *cursor;
+  int stride, i, n_channels;
+  unsigned char *pixels, *end, *argb_pixels, *s, *d;
 
-  backend_wayland->cursor_theme =
-    wl_cursor_theme_load (NULL, /* default */
-                          32,
-                          backend_wayland->wayland_shm);
+  stride = gdk_pixbuf_get_rowstride (pixbuf);
+  pixels = gdk_pixbuf_get_pixels (pixbuf);
+  n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+  argb_pixels = map;
 
-  cursor = wl_cursor_theme_get_cursor (backend_wayland->cursor_theme,
-                                       "left_ptr");
+#define MULT(_d,c,a,t) \
+  do { t = c * a + 0x7f; _d = ((t >> 8) + t) >> 8; } while (0)
 
-  backend_wayland->cursor_buffer =
-    wl_cursor_image_get_buffer (cursor->images[0]);
+  if (n_channels == 4)
+    {
+      for (i = 0; i < height; i++)
+        {
+          s = pixels + i * stride;
+          end = s + width * 4;
+          d = argb_pixels + i * width * 4;
+          while (s < end)
+            {
+              unsigned int t;
+
+              MULT(d[0], s[2], s[3], t);
+              MULT(d[1], s[1], s[3], t);
+              MULT(d[2], s[0], s[3], t);
+              d[3] = s[3];
+              s += 4;
+              d += 4;
+            }
+        }
+    }
+  else if (n_channels == 3)
+    {
+      for (i = 0; i < height; i++)
+        {
+          s = pixels + i * stride;
+          end = s + width * 3;
+          d = argb_pixels + i * width * 4;
+          while (s < end)
+            {
+              d[0] = s[2];
+              d[1] = s[1];
+              d[2] = s[0];
+              d[3] = 0xff;
+              s += 3;
+              d += 4;
+            }
+        }
+    }
+}
+
+static struct wl_buffer *
+create_cursor (ClutterBackendWayland *backend_wayland,
+               GdkPixbuf             *pixbuf)
+{
+  int stride, fd;
+  char *filename;
+  GError *error = NULL;
+  struct wl_buffer *buffer;
+  gint width, height;
+  gsize size;
+  unsigned char *map;
+
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
+  stride = width * 4;
+  size = stride * height;
+
+  fd = g_file_open_tmp ("wayland-shm-XXXXXX", &filename, &error);
+  if (fd < 0)
+    {
+      g_critical (G_STRLOC ": Opening temporary file failed: %s", error->message);
+      g_error_free (error);
+      return NULL;
+    }
+
+  unlink (filename);
+  g_free (filename);
+
+  if (ftruncate (fd, size) < 0)
+    {
+      g_critical (G_STRLOC ": Setting the size of temporary file failed: %s", g_strerror (errno));
+      close (fd);
+      return NULL;
+    }
+
+  map = mmap (NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+
+  if (map == MAP_FAILED)
+    {
+      g_critical (G_STRLOC ": Memory mapping file failed: %s", g_strerror (errno));
+      close (fd);
+      return NULL;
+   }
+
+  set_pixbuf (pixbuf, map, width, height);
+
+  buffer = wl_shm_create_buffer (backend_wayland->wayland_shm,
+                                 fd,
+                                 width,
+                                 height,
+                                 stride,
+                                 WL_SHM_FORMAT_ARGB32);
+
+  close(fd);
+  munmap (map, size);
+
+  return buffer;
+}
+
+static void
+clutter_backend_wayland_load_cursor (ClutterBackendWayland *backend_wayland)
+{
+  const gchar * const *directories;
+  gint j;
+  GdkPixbuf *pixbuf = NULL;
+  GError *error = NULL;
+
+  directories = g_get_system_data_dirs();
+
+  for (j = 0; directories[j] != NULL; j++)
+    {
+      gchar *filename;
+      filename = g_build_filename (directories[j],
+                                   "weston",
+                                   "left_ptr.png",
+                                   NULL);
+      if (g_file_test (filename, G_FILE_TEST_EXISTS))
+        {
+          pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+
+          if (error != NULL)
+            {
+              g_warning ("Failed to load cursor: %s: %s",
+                         filename, error->message);
+              g_error_free (error);
+              return;
+            }
+          break;
+        }
+    }
+
+  if (!pixbuf)
+    return;
+
+  backend_wayland->cursor_buffer = create_cursor (backend_wayland, pixbuf);
 
   if (backend_wayland->cursor_buffer)
     {
-      backend_wayland->cursor_x = cursor->images[0]->hotspot_x;
-      backend_wayland->cursor_y = cursor->images[0]->hotspot_y;
+      backend_wayland->cursor_x = 15;
+      backend_wayland->cursor_y = 15;
     }
 
-  backend_wayland->cursor_surface =
-    wl_compositor_create_surface (backend_wayland->wayland_compositor);
+  g_object_unref (pixbuf);
 }
 
 static void
 clutter_backend_wayland_init (ClutterBackendWayland *backend_wayland)
 {
 }
-
-/**
- * clutter_wayland_set_display
- * @display: pointer to a wayland display
- *
- * Sets the display connection Clutter should use; must be called
- * before clutter_init(), clutter_init_with_args() or other functions
- * pertaining Clutter's initialization process.
- *
- * If you are parsing the command line arguments by retrieving Clutter's
- * #GOptionGroup with clutter_get_option_group() and calling
- * g_option_context_parse() yourself, you should also call
- * clutter_wayland_set_display() before g_option_context_parse().
- *
- * Since: 1.16
- */
-void
-clutter_wayland_set_display (struct wl_display *display)
-{
-  if (_clutter_context_is_initialized ())
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  _foreign_display = display;
-}
-
-/**
- * clutter_wayland_disable_event_retrieval:
- *
- * Disables the dispatch of the events in the main loop.
- *
- * This is useful for integrating Clutter with another library that will do the
- * event dispatch; in general only a single source should be acting on changes
- * on the Wayland file descriptor.
- *
- * This function can only be called before calling clutter_init().
- *
- * This function should not be normally used by applications.
- *
- * Since: 1.16
- */
-void
-clutter_wayland_disable_event_retrieval (void)
-{
-  if (_clutter_context_is_initialized ())
-    {
-      g_warning ("%s() can only be used before calling clutter_init()",
-                 G_STRFUNC);
-      return;
-    }
-
-  _no_event_dispatch = TRUE;
-}
