@@ -44,13 +44,12 @@ struct _ClutterKeymapX11
 
   ClutterBackend *backend;
 
-  int min_keycode;
-  int max_keycode;
+  gint min_keycode;
+  gint max_keycode;
 
   ClutterModifierType modmap[8];
 
   ClutterModifierType num_lock_mask;
-  ClutterModifierType scroll_lock_mask;
 
 #ifdef HAVE_XKB
   XkbDescPtr xkb_desc;
@@ -181,10 +180,6 @@ get_xkb (ClutterKeymapX11 *keymap_x11)
     keymap_x11->num_lock_mask = XkbKeysymToModifiers (backend_x11->xdpy,
                                                       XK_Num_Lock);
 
-  if (keymap_x11->scroll_lock_mask == 0)
-    keymap_x11->scroll_lock_mask = XkbKeysymToModifiers (backend_x11->xdpy,
-                                                         XK_Scroll_Lock);
-
   return keymap_x11->xkb_desc;
 }
 #endif /* HAVE_XKB */
@@ -365,7 +360,6 @@ clutter_keymap_x11_translate_event (ClutterEventTranslator *translator,
           retval = CLUTTER_TRANSLATE_REMOVE;
           break;
 
-        case XkbNewKeyboardNotify:
         case XkbMapNotify:
           CLUTTER_NOTE (EVENT, "Updating keyboard mapping");
           XkbRefreshKeyboardMapping (&xkb_event->map);
@@ -415,38 +409,14 @@ _clutter_keymap_x11_get_caps_lock_state (ClutterKeymapX11 *keymap)
   return keymap->caps_lock_state;
 }
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-
-/* XXX - yes, I know that XKeycodeToKeysym() has been deprecated; hopefully,
- * this code will never get run on any decent system that is also able to
- * run Clutter. I just don't want to copy the implementation inside GDK for
- * a fallback path.
- */
-static int
-translate_keysym (ClutterKeymapX11 *keymap,
-                  guint             hardware_keycode)
-{
-  ClutterBackendX11 *backend_x11;
-  gint retval;
-
-  backend_x11 = CLUTTER_BACKEND_X11 (keymap->backend);
-
-  retval = XKeycodeToKeysym (backend_x11->xdpy, hardware_keycode, 0);
-
-  return retval;
-}
-
-G_GNUC_END_IGNORE_DEPRECATIONS
-
 gint
 _clutter_keymap_x11_translate_key_state (ClutterKeymapX11    *keymap,
                                          guint                hardware_keycode,
-                                         ClutterModifierType *modifier_state_p,
+                                         ClutterModifierType  modifier_state,
                                          ClutterModifierType *mods_p)
 {
   ClutterBackendX11 *backend_x11;
   ClutterModifierType unconsumed_modifiers = 0;
-  ClutterModifierType modifier_state = *modifier_state_p;
   gint retval;
 
   g_return_val_if_fail (CLUTTER_IS_KEYMAP_X11 (keymap), 0);
@@ -470,21 +440,17 @@ _clutter_keymap_x11_translate_key_state (ClutterKeymapX11    *keymap,
     }
   else
 #endif /* HAVE_XKB */
-    retval = translate_keysym (keymap, hardware_keycode);
+    retval = XKeycodeToKeysym (backend_x11->xdpy, hardware_keycode, 0);
 
   if (mods_p)
     *mods_p = unconsumed_modifiers;
 
-  *modifier_state_p = modifier_state & ~(keymap->num_lock_mask |
-                                         keymap->scroll_lock_mask |
-                                         LockMask);
-
   return retval;
 }
 
 gboolean
 _clutter_keymap_x11_get_is_modifier (ClutterKeymapX11 *keymap,
-                                     gint              keycode)
+                                     guint             keycode)
 {
   g_return_val_if_fail (CLUTTER_IS_KEYMAP_X11 (keymap), FALSE);
 
