@@ -85,7 +85,7 @@ typedef struct _ClutterX11XInputDevice ClutterX11XInputDevice;
  * ClutterX11FilterFunc:
  * @xev: Native X11 event structure
  * @cev: Clutter event structure
- * @data: (closure): user data passed to the filter function
+ * @data: user data passed to the filter function
  *
  * Filter function for X11 native events.
  *
@@ -97,79 +97,53 @@ typedef ClutterX11FilterReturn (*ClutterX11FilterFunc) (XEvent        *xev,
                                                         ClutterEvent  *cev,
                                                         gpointer       data);
 
-CLUTTER_AVAILABLE_IN_ALL
 void     clutter_x11_trap_x_errors       (void);
-CLUTTER_AVAILABLE_IN_ALL
 gint     clutter_x11_untrap_x_errors     (void);
 
-CLUTTER_AVAILABLE_IN_ALL
 Display *clutter_x11_get_default_display (void);
-CLUTTER_AVAILABLE_IN_ALL
 int      clutter_x11_get_default_screen  (void);
-CLUTTER_AVAILABLE_IN_ALL
 Window   clutter_x11_get_root_window     (void);
-CLUTTER_AVAILABLE_IN_ALL
 XVisualInfo *clutter_x11_get_visual_info (void);
-CLUTTER_AVAILABLE_IN_ALL
 void     clutter_x11_set_display         (Display * xdpy);
 
+#ifndef CLUTTER_DISABLE_DEPRECATED
 CLUTTER_DEPRECATED_FOR(clutter_x11_get_visual_info)
 XVisualInfo *clutter_x11_get_stage_visual  (ClutterStage *stage);
+#endif
 
-CLUTTER_AVAILABLE_IN_ALL
 Window       clutter_x11_get_stage_window  (ClutterStage *stage);
-CLUTTER_AVAILABLE_IN_ALL
 gboolean     clutter_x11_set_stage_foreign (ClutterStage *stage,
                                             Window        xwindow);
 
-CLUTTER_AVAILABLE_IN_ALL
 void         clutter_x11_add_filter    (ClutterX11FilterFunc func,
                                         gpointer             data);
-CLUTTER_AVAILABLE_IN_ALL
 void         clutter_x11_remove_filter (ClutterX11FilterFunc func,
                                         gpointer             data);
 
-CLUTTER_AVAILABLE_IN_ALL
 ClutterX11FilterReturn clutter_x11_handle_event (XEvent *xevent);
 
-CLUTTER_AVAILABLE_IN_ALL
 void     clutter_x11_disable_event_retrieval (void);
-CLUTTER_AVAILABLE_IN_ALL
 gboolean clutter_x11_has_event_retrieval (void);
 
-CLUTTER_AVAILABLE_IN_ALL
 ClutterStage *clutter_x11_get_stage_from_window (Window win);
 
+#ifndef CLUTTER_DISABLE_DEPRECATED
 CLUTTER_DEPRECATED_FOR(clutter_device_manager_peek_devices)
 const GSList* clutter_x11_get_input_devices (void);
+#endif
 
-CLUTTER_DEPRECATED_IN_1_14
 void     clutter_x11_enable_xinput (void);
-CLUTTER_AVAILABLE_IN_ALL
 gboolean clutter_x11_has_xinput (void);
 
-CLUTTER_AVAILABLE_IN_ALL
 gboolean clutter_x11_has_composite_extension (void);
 
-CLUTTER_AVAILABLE_IN_ALL
 void     clutter_x11_set_use_argb_visual (gboolean use_argb);
-CLUTTER_AVAILABLE_IN_ALL
 gboolean clutter_x11_get_use_argb_visual (void);
 
-CLUTTER_AVAILABLE_IN_1_22
-void     clutter_x11_set_use_stereo_stage (gboolean use_stereo);
-CLUTTER_AVAILABLE_IN_1_22
-gboolean clutter_x11_get_use_stereo_stage (void);
-
-CLUTTER_AVAILABLE_IN_ALL
 Time clutter_x11_get_current_event_time (void);
 
-CLUTTER_AVAILABLE_IN_ALL
 gint clutter_x11_event_get_key_group (const ClutterEvent *event);
 
-CLUTTER_AVAILABLE_IN_ALL
-guint clutter_x11_event_sequence_get_touch_detail (const ClutterEventSequence *sequence);
-
 G_END_DECLS
 
 #endif /* __CLUTTER_X11_H__ */
