@@ -128,11 +128,20 @@ main (int argc, char **argv)
   /* sanity check for the test suite itself */
   TEST_CONFORM_TODO ("/suite", verify_failure);
 
+  TEST_CONFORM_SIMPLE ("/actor", actor_add_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_insert_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_raise_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_lower_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_replace_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_remove_child);
+  TEST_CONFORM_SIMPLE ("/actor", actor_remove_all);
   TEST_CONFORM_SIMPLE ("/actor", actor_destruction);
   TEST_CONFORM_SIMPLE ("/actor", actor_anchors);
   TEST_CONFORM_SIMPLE ("/actor", actor_picking);
   TEST_CONFORM_SIMPLE ("/actor", actor_fixed_size);
   TEST_CONFORM_SIMPLE ("/actor", actor_preferred_size);
+  TEST_CONFORM_SIMPLE ("/actor", basic_layout);
+  TEST_CONFORM_SIMPLE ("/actor", margin_layout);
   TEST_CONFORM_SIMPLE ("/actor", test_offscreen_redirect);
   TEST_CONFORM_SIMPLE ("/actor", test_shader_effect);
 
@@ -208,6 +217,7 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/script", test_script_layout_property);
 
   TEST_CONFORM_SIMPLE ("/timeline", test_timeline);
+  TEST_CONFORM_SIMPLE ("/timeline", markers_from_script);
   TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", timeline_interpolation);
   TEST_CONFORM_SKIP (!g_test_slow (), "/timeline", timeline_rewind);
 
